/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2004 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "StdAfx.h"
#include "SolidTexture.h"

SolidTexture::SolidTexture()
{
}

SolidTexture::~SolidTexture()
{
	Gradient.clear();
	HoverGradient.clear();
	PressedGradient.clear();
}

void SolidTexture::configure(const string &prefix, const string &subKey)
{
	bevelSize = GetxRCInt(prefix, "SolidBevelSize", GetxRCInt(subKey, "SolidBevelSize", GetxRCInt("AllLabels", "SolidBevelSize", 0, 0), 0), 0);
	hoverbevelSize = GetxRCInt(prefix, "HoverSolidBevelSize", GetxRCInt(subKey, "HoverSolidBevelSize", GetxRCInt("AllLabels", "HoverSolidBevelSize", bevelSize, 0), 0), 0);
	pressedbevelSize = GetxRCInt(prefix, "PressedSolidBevelSize", GetxRCInt(subKey, "PressedSolidBevelSize", GetxRCInt("AllLabels", "PressedSolidBevelSize", hoverbevelSize, 0), 0), 0);

	string solidPaintingColors = "";
	solidPaintingColors = GetxRCLine(prefix, "SolidColors", GetxRCLine(subKey, "SolidColors", GetxRCLine("AllLabels", "SolidColors", "ffffff")));
	parseCFG(solidPaintingColors, backgroundColor, lightBevelColor, darkBevelColor);
	//BG Insert in Gradient List
	Gradient.insert(Gradient.end(), backgroundColor );

	gradientColor = GetxRCLine(prefix, "SolidGradientColors", GetxRCLine(subKey, "SolidGradientColors", GetxRCLine("AllLabels", "SolidGradientColors", "")));
	parseGradientCFG( gradientColor, 0 );

	verticalGradient = GetxRCBool(prefix, "SolidGradientVertical", GetxRCBool(subKey, "SolidGradientVertical", GetxRCBool("AllLabels", "SolidGradientVertical")));

	solidPaintingColors = GetxRCLine(prefix, "HoverSolidColors", GetxRCLine(subKey, "HoverSolidColors", GetxRCLine("AllLabels", "HoverSolidColors", "")));
	if (!solidPaintingColors.empty())
		parseCFG(solidPaintingColors, hoverbackgroundColor, hoverlightBevelColor, hoverdarkBevelColor);
	else
	{
		hoverbackgroundColor = backgroundColor;
		hoverdarkBevelColor = darkBevelColor;
		hoverlightBevelColor = lightBevelColor;
	}
	//BG Insert in Gradient List
	HoverGradient.insert(HoverGradient.end(), hoverbackgroundColor );

	hovergradientColor = GetxRCLine(prefix, "HoverSolidGradientColors", GetxRCLine(subKey, "HoverSolidGradientColors", GetxRCLine("AllLabels", "HoverSolidGradientColors", gradientColor)));
	parseGradientCFG( hovergradientColor, 1 );

	hoververticalGradient = GetxRCBool(prefix, "HoverSolidGradientVertical", GetxRCBool(subKey, "HoverSolidGradientVertical", GetxRCBool("AllLabels", "HoverSolidGradientVertical", verticalGradient)));

	solidPaintingColors = GetxRCLine(prefix, "PressedSolidColors", GetxRCLine(subKey, "PressedSolidColors", GetxRCLine("AllLabels", "PressedSolidColors", "")));
	if (!solidPaintingColors.empty())
		parseCFG(solidPaintingColors, pressedbackgroundColor, pressedlightBevelColor, presseddarkBevelColor);
	else
	{
		pressedbackgroundColor = hoverbackgroundColor;
		presseddarkBevelColor = hoverdarkBevelColor;
		pressedlightBevelColor = hoverlightBevelColor;
	}
	//BG Insert in Gradient List
	PressedGradient.insert(PressedGradient.end(), pressedbackgroundColor );

	pressedgradientColor = GetxRCLine(prefix, "PressedSolidGradientColors", GetxRCLine(subKey, "PressedSolidGradientColors", GetxRCLine("AllLabels", "PressedSolidGradientColors", hovergradientColor)));
	parseGradientCFG( pressedgradientColor, 2 );

	pressedverticalGradient = GetxRCBool(prefix, "PressedSolidGradientVertical", GetxRCBool(subKey, "PressedSolidGradientVertical", GetxRCBool("AllLabels", "PressedSolidGradientVertical", hoververticalGradient)));
}

void SolidTexture::parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor)
{
	int color;
	char main[32], light[32], dark[32];
	char *buffers[] = {main, light, dark};

	int numTokens = LCTokenize(cfg.c_str(), buffers, 3, NULL);
		
	if(numTokens == 1)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 2)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 3)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(dark, NULL, 16);
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
}

void SolidTexture::parseGradientCFG(string cfg, int imageType)
{
	int color;
	COLORREF insertColor;
	char token[MAX_LINE_LENGTH];
	LPCSTR nextToken = cfg.c_str();
	
	int counter = 1;
	while (GetToken(nextToken, token, &nextToken, false))
	{
		color = strtol(token, NULL, 16);
		insertColor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		if (imageType == 0)
		{
			Gradient.insert(Gradient.end(), insertColor );
		}
		else if (imageType == 1)
		{
			HoverGradient.insert(HoverGradient.end(), insertColor );
		}
		else if (imageType == 2)
		{
			PressedGradient.insert(PressedGradient.end(), insertColor );
		}
		counter++;
	}
}

void SolidTexture::apply(HDC hDC, int x, int y, int width, int height, int transparencyMode, int imageType)
{
	RECT r;
	r.left = x;
	r.top = y;
	r.right = r.left + width;
	r.bottom = r.top + height;

	if (imageType == 0)
	{
		// fill in the background
		if (bevelSize > 0 && (bevelSize*2 < width && bevelSize*2 < height))
		{
			::SetBkColor(hDC, darkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-bevelSize;
			r.bottom = r.bottom-bevelSize;
			::SetBkColor(hDC, lightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+bevelSize;
			r.top = r.top+bevelSize;

			if ( Gradient.size() > 1 )
				solidGradientPaint(hDC, Gradient, r, verticalGradient);
			else
			{
				::SetBkColor(hDC, backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( Gradient.size() > 1 )
				solidGradientPaint(hDC, Gradient, r, verticalGradient);
			else
			{
				::SetBkColor(hDC, backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if (imageType == 1)
	{
		// fill in the background
		if (hoverbevelSize > 0 && (hoverbevelSize*2 < width && hoverbevelSize*2 < height))
		{
			::SetBkColor(hDC, hoverdarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-hoverbevelSize;
			r.bottom = r.bottom-hoverbevelSize;
			::SetBkColor(hDC, hoverlightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+hoverbevelSize;
			r.top = r.top+hoverbevelSize;

			if ( HoverGradient.size() > 1 )
				solidGradientPaint(hDC, HoverGradient, r, hoververticalGradient);
			else
			{
				::SetBkColor(hDC, hoverbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( HoverGradient.size() > 1 )
				solidGradientPaint(hDC, HoverGradient, r, hoververticalGradient);
			else
			{
				::SetBkColor(hDC, hoverbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if (imageType == 2)
	{
		// fill in the background
		if (pressedbevelSize > 0 && (pressedbevelSize*2 < width && pressedbevelSize*2 < height))
		{
			::SetBkColor(hDC, presseddarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-pressedbevelSize;
			r.bottom = r.bottom-pressedbevelSize;
			::SetBkColor(hDC, pressedlightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+pressedbevelSize;
			r.top = r.top+pressedbevelSize;

			if ( PressedGradient.size() > 1 )
				solidGradientPaint(hDC, PressedGradient, r, pressedverticalGradient);
			else
			{
				::SetBkColor(hDC, pressedbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( PressedGradient.size() > 1 )
				solidGradientPaint(hDC, PressedGradient, r, pressedverticalGradient);
			else
			{
				::SetBkColor(hDC, pressedbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
}

void SolidTexture::solidGradientPaint(HDC destHDC, vector<COLORREF> workGradient, RECT r, bool vertical)
{
	int nWidth = r.right - r.left;
	int nHeight = r.bottom - r.top;

	RECT r2;
	double steps = 1;
	double cr, cg, cb, dr, dg, db;
	int gradSteps;

	int colorCount = workGradient.size();

	if (vertical)
		gradSteps = max(nHeight / (colorCount-1), 1);
	else
		gradSteps = max(nWidth / (colorCount-1), 1);

	int lastGradientPos;
	if (vertical)
		lastGradientPos = r.top;
	else
		lastGradientPos = r.left;

	for (ColorRefIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
	{
		if (cit == workGradient.end())
			break;

		COLORREF one = *cit;
		cit++;
		COLORREF two = *cit;
		cit--;
	
		// get current color
		cr = GetRValue( one );
		cg = GetGValue( one );
		cb = GetBValue( one );

		// Get delta color
		dr = (cr - GetRValue( two )) / gradSteps;
		dg = (cg - GetGValue( two )) / gradSteps;
		db = (cb - GetBValue( two )) / gradSteps;

		for (int i = 1; i <= gradSteps; i++)
		{
			cr -= dr;
			cg -= dg;
			cb -= db;

			if (vertical)
				SetRect(&r2, r.left, (int)(lastGradientPos+steps*(i - 1)), r.right, min((int)(lastGradientPos+steps*i), r.bottom));
			else
				SetRect(&r2, (int)(lastGradientPos+steps*(i - 1)), r.top, min((int)(lastGradientPos+steps*i), r.right), r.bottom);

			if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
			{
				::SetBkColor(destHDC, RGB((int)cr, (int)cg, (int)cb));
				::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
			}
		}
		if (vertical)
			lastGradientPos = r2.bottom;
		else
			lastGradientPos = r2.right;
	}
}
