/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2004 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "StdAfx.h"
#include "Utility.h"

#include "BlankTexture.h"
#include "ImageTexture.h"
#include "SolidTexture.h"

#include "xFont.h"

HWND FindWindowEx2(string pszClass, string pszTitle)
{
	HWND hWnd = NULL;
	FindWindowEx2Data fwed;
	fwed.phWnd = &hWnd;
	fwed.pszClass = pszClass.c_str();
	fwed.pszTitle = pszTitle.c_str();
	EnumWindows(FindWindowEx2EnumWindowsProc, (LPARAM)&fwed);
	return *(fwed.phWnd);
}

BOOL CALLBACK FindWindowEx2EnumWindowsProc(HWND hWnd, LPARAM lParam)
{
	FindWindowEx2Data* pfwed = (FindWindowEx2Data*)lParam;
	char szClass[MAX_PATH_LENGTH];
	char szTitle[MAX_PATH_LENGTH];
	if(pfwed->pszClass)
	{
		GetClassName(hWnd, szClass, MAX_PATH_LENGTH);
	}
	if(pfwed->pszTitle)
	{
		GetWindowText(hWnd, szTitle, MAX_PATH_LENGTH);
	}
	if((pfwed->pszClass != NULL ? !stricmp(szClass, pfwed->pszClass) : TRUE) && (pfwed->pszTitle != NULL ? !stricmp(szTitle, pfwed->pszTitle) : TRUE))
	{
		*(pfwed->phWnd) = hWnd;
		return FALSE;
	}
	EnumChildWindows(hWnd, FindWindowEx2EnumWindowsProc, lParam);
	return (*pfwed->phWnd == NULL);
}

HINSTANCE xLSExecute(HWND Owner, LPCSTR szCommand, int nShowCmd)
{
	CString process = szCommand;
	process.MakeLower();
	if (process.Find( "http:" ) != -1 && process.Find( "!" ) == -1)
	{
		SHELLEXECUTEINFO ShExecInfo = {0};
		ShExecInfo.cbSize = sizeof(SHELLEXECUTEINFO);
		ShExecInfo.fMask = SEE_MASK_NOCLOSEPROCESS;
		ShExecInfo.hwnd = NULL;
		ShExecInfo.lpVerb = "";
		ShExecInfo.lpFile = szCommand;		
		ShExecInfo.lpParameters = "";	
		ShExecInfo.lpDirectory = NULL;
		ShExecInfo.nShow = nShowCmd;
		ShExecInfo.hInstApp = NULL;	
		ShellExecuteEx(&ShExecInfo);

		return ShExecInfo.hInstApp;
	}
	else
	{
		return LSExecute(Owner, szCommand, nShowCmd);
	}
}

void PrefixedLSSetVariable(string prefix, string var, string value)
{
	if ( !var.empty() )
		prefix.append(var);

	LSSetVariable(prefix.c_str(), value.c_str());
}

bool IsOS(const int &os)
{	
	static int osversion = 0;

	static bool bInitialized = false;

	if (!bInitialized)
	{
		OSVERSIONINFO verInfo;
		verInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		GetVersionEx(&verInfo);
		
		switch(verInfo.dwPlatformId)
		{
			case VER_PLATFORM_WIN32_WINDOWS:

				osversion |= OS_9X;
			
				if(verInfo.dwMinorVersion >= 90)
				{
					osversion |= OS_ME;
				}
				else if(verInfo.dwMinorVersion >= 10)
				{
					osversion |= OS_98;
				}
				else
				{
					osversion |= OS_95;
				}
				break;
			
			case VER_PLATFORM_WIN32_NT:
			
				osversion |= OS_NT;
			
				if(verInfo.dwMajorVersion >= 5)
				{
					osversion |= OS_2KXP;

					if (verInfo.dwMinorVersion >= 1)
					{
						osversion |= OS_XP;
					}
					else
					{
						osversion |= OS_2K;
					}
				}
				else
				{
					osversion |= OS_NT4;
				}
				break;

			default:;
		}
		bInitialized = true;
	}

	return (osversion & os) ? true : false;
}

string QueryPixelDepth()
{
	CString Response = "";
	DEVMODE DevMode;
	DevMode.dmSize=sizeof(DEVMODE);
	
	if (EnumDisplaySettings(NULL,ENUM_CURRENT_SETTINGS,&DevMode))
	{
		Response.Format("%ld", DevMode.dmBitsPerPel);
	}
	else
		Response = "8";
	
	return (string)Response;
}

// trim leading and trailing whitespace
string trim(const string &toTrim)
{
	int start = 0;
	int end = toTrim.length() - 1;
	
	while(start <= end && isspace(toTrim[start]))
		start++;
	
	while(start <= end && isspace(toTrim[end]))
		end--;
	
	return toTrim.substr(start, end - start + 1);
}

// retrieve handle of the desktop window
HWND GetLitestepDesktop()
{
	HWND hWnd = FindWindow("DesktopBackgroundClass", NULL);

	if(hWnd)
		return hWnd;

	return GetDesktopWindow();
}

// is the given window a toplevel application window?
bool IsAppWindow(const HWND &hWnd)
{
	if(!IsWindowVisible(hWnd))
		return false;

	if(GetWindow(hWnd, GW_OWNER) != 0)
		return false;

	long exStyle = GetWindowLong(hWnd, GWL_EXSTYLE);

	if(exStyle & WS_EX_APPWINDOW)
		return true;

	if(exStyle & WS_EX_TOOLWINDOW)
		return false;

	if(GetWindowLong(hWnd, GWL_USERDATA) == magicDWord)
		return false;

	return true;
}

// modify window style
void ModifyStyle(HWND hWnd, DWORD removeStyle, DWORD addStyle)
{
	SetWindowLong(hWnd, GWL_STYLE, (GetWindowLong(hWnd, GWL_STYLE) & ~removeStyle) | addStyle);
}

void ModifyStyleEx(HWND hWnd, DWORD removeStyle, DWORD addStyle)
{
	SetWindowLong(hWnd, GWL_EXSTYLE, (GetWindowLong(hWnd, GWL_EXSTYLE) & ~removeStyle) | addStyle);
}

int SplitString(const string &process, string &one, string &two, string &three, string &four, string &five)
{
	int count = 0;
	one = "";
	two = "";
	three = "";
	four = "";
	five = "";
	char cone[256] = {0};
	char ctwo[256] = {0};
	char cthree[256] = {0};
	char cfour[256] = {0};
	char cfive[256] = {0};
	char *cbuffers[] = {cone, ctwo, cthree, cfour, cfive};
		
	int numTokens = LCTokenize( process.c_str(), cbuffers, 5, NULL);
	if( numTokens >= 1 )
	{
		one = cone;
		count++;
		
		if ( numTokens >= 2 )
		{
			two = ctwo;
			count++;
		}
		
		if ( numTokens >= 3 )
		{
			three = cthree;
			count++;
		}
		
		if ( numTokens >= 4 )
		{
			four = cfour;
			count++;
		}

		if ( numTokens == 5 )
		{
			five = cfive;
			count++;
		}
	}

	return count;
}

// retrieve a boolean value from the step.rc
bool GetxRCBool(const string &object, const string &subKey, bool defaultVal)
{
	string key = object + subKey;
	return GetRCBoolDef(key.c_str(), defaultVal) ? true : false;
}

// retrieve a color value from step.rc
int GetxRCColor(const string &object, const string &subKey, int defaultVal)
{
	string key = object + subKey;
	return GetRCColor(key.c_str(), defaultVal);
}

// parse a coordinate specification
int xParseCoord(const string &aString, int defaultVal, int maxVal)
{
	string strVal = trim(aString);

	if (strVal.length() == 0)
		return defaultVal;

	bool negative = false;
	bool realnegative = false;
	bool center = false;
	UINT i = 0;
	int value = 0;

	if (strVal[i] == '-')
	{
		negative = true;
		i++;
	}
	else if (strVal[i] == '~')
	{
		realnegative = true;
		i++;
	}
	else if (strVal[i] == '+')
	{
		i++;
	}

	while (i < strVal.length())
	{
		if(strVal[i] >= '0' && strVal[i] <= '9')
		{
			value = (value * 10) + (strVal[i] - '0');
		}
		else
		{
			if(strVal[i] == 'c' || strVal[i] == 'C')
				center = true;

			break;
		}

		i++;
	}

	if (center)
	{
		if (realnegative)
			negative = true;

		if (negative)
			value = (maxVal / 2) - value;
		else
			value = (maxVal / 2) + value;
	}
	else if (realnegative)
	{
		value = -value;
	}
	else if (negative)
	{
		value = maxVal - value;
	}

	return value;
}

// retrieve a coordinate from step.rc, accounting for negative values and centering
int GetxRCCoord(const string &prefix, const string &baseName, int defaultVal, int maxVal)
{
	string strVal = GetxRCString(prefix, baseName, "");
	return xParseCoord(strVal, defaultVal, maxVal);
}

// parse a dimension
int xParseDimen(const string &aString, int defaultVal, int maxVal)
{
	string strVal = trim(aString);

	if (strVal.length() == 0)
		return defaultVal;

	bool negative = false;
	bool percentage = false;
	UINT i = 0;
	int value = 0;

	if (strVal[i] == '-')
	{
		negative = true;
		i++;
	}

	while (i < strVal.length())
	{
		if (strVal[i] >= '0' && strVal[i] <= '9')
		{
			value = (value * 10) + (strVal[i] - '0');
		}
		else
		{
			if (strVal[i] == '%')
				percentage = true;

			break;
		}

		i++;
	}

	if (percentage)
		value = (value * maxVal) / 100;

	if (negative)
		value = maxVal - value;

	return value;
}

// retrieve a dimension (absolute value or percentage) from step.rc
int GetxRCDimen(const string &prefix, const string &baseName, int defaultVal, int maxVal)
{
	string strVal = GetxRCString(prefix, baseName, "");
	return xParseDimen(strVal, defaultVal, maxVal);
}

// retrieve a font from step.rc
xFont *GetRCFont(const string &object, xFont *defaultVal)
{
	xFont *font = new xFont;
	font->configure(object, defaultVal);
	return font;
}

// retrieve a list of names from step.rc
StringList GetxRCNameList(const string &prefix)
{
	StringList resultList;
	FILE * f = LCOpen(NULL);

	if(f != NULL)
	{
		char lineBuffer[MAX_LINE_LENGTH];

		while(LCReadNextConfig(f, prefix.c_str(), lineBuffer, MAX_LINE_LENGTH))
		{
			char temp[64], name[64];
			char *buffers[] = {temp, name};
			
			if(LCTokenize(lineBuffer, buffers, 2, NULL) >= 2)
			{
				resultList.insert(resultList.end(), name);
			}
		}

		LCClose(f);
	}

	return resultList;
}

StringList GetxRCLineList(const string &prefix)
{
	StringList resultList;
	FILE * f = LCOpen(NULL);

	if(f != NULL)
	{
		char lineBuffer[MAX_LINE_LENGTH];

		while(LCReadNextConfig(f, prefix.c_str(), lineBuffer, MAX_LINE_LENGTH))
		{
			char temp[64], configline[MAX_LINE_LENGTH];
			char *buffers[] = {temp};
			
			if(LCTokenize(lineBuffer, buffers, 1, configline) >= 1)
			{
				resultList.insert(resultList.end(), configline);
			}
		}
		
		LCClose(f);
	}

	return resultList;
}

// retrieve a named value from step.rc
int GetxRCNamedValue(const string &prefix, const string &baseName, const NameValuePair *nameValuePairs, int defaultVal)
{
	string name = GetxRCString(prefix, baseName, "");
	const NameValuePair *current = nameValuePairs;

	if(name.length() > 0)
	{
		while(current->name)
		{
			if( stricmp(current->name, name.c_str()) == NULL )
				return current->value;

			current++;
		}
	}

	return defaultVal;
}

// retrieve an integer value from step.rc, make sure it is between minVal and maxVal inclusive
int GetxRCInt(const string &object, const string &subKey, int defaultVal, int minVal, int maxVal)
{
	string key = object + subKey;
	int answer = GetRCInt(key.c_str(), defaultVal);
	return max(min(answer, maxVal), minVal);
}

// retrieve a line from step.rc
string GetxRCLine(const string &object, const string &subKey, const string &defaultVal)
{
	string key = object + subKey;
	char buffer[MAX_LINE_LENGTH];
	GetRCLine(key.c_str(), buffer, MAX_LINE_LENGTH, defaultVal.c_str());
	return string(buffer);
}

// retrieve a string value from step.rc
string GetxRCString(const string &object, const string &subKey, const string &defaultVal)
{
	string key = object + subKey;
	char buffer[MAX_LINE_LENGTH];
	GetRCString(key.c_str(), buffer, defaultVal.c_str(), MAX_LINE_LENGTH);
	return string(buffer);
}

// retrieve a texture from step.rc
Texture* GetRCTexture(const string &object, const string &subKey, Texture* defaultVal)
{
	Texture *texture = 0;

	//Comparison which mode is to be used
	int useImage = 0;
	if ( !(GetxRCString(object, "Image", "")).empty() )
		useImage = 1;
	else if ( !(GetxRCString(subKey, "Image", "")).empty() )
		useImage = 2;
	else if ( !(GetxRCString("AllLabels", "Image", "")).empty() )
		useImage = 3;

	if (useImage > 1 && !(GetxRCString(object, "SolidColors", "")).empty() )
		useImage = -1;
	else if (useImage > 2 && !(GetxRCString(subKey, "SolidColors", "")).empty() )
		useImage = -1;

	if( useImage > 0 )
	{
		texture = new ImageTexture();
		texture->configure(object, subKey);
	}
	else if( !(GetxRCString(object, "SolidColors", "")).empty() || !(GetxRCString(subKey, "SolidColors", "")).empty() || !(GetxRCString("AllLabels", "SolidColors", "")).empty())
	{
		texture = new SolidTexture();
		texture->configure(object, subKey);
	}
	else if(defaultVal)
	{
		texture = defaultVal;
	}
	else
	{
		texture = new BlankTexture();
		texture->configure(object, subKey);
	}

	return texture;
}

HDC hdcDesktop = 0;
HBITMAP hbmDesktop = 0;

// enhanced version of PaintDesktop
void PaintDesktopEx(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, int xSrc, int ySrc, BOOL updateCache)
{
	if(hdcDesktop == 0 || updateCache)
	{
		if(hdcDesktop == 0 && hdcDest == 0)
			return;

		if(hbmDesktop != 0)
		{
			hbmDesktop = (HBITMAP) SelectObject(hdcDesktop, hbmDesktop);
			DeleteObject(hbmDesktop);
			hbmDesktop = 0;
		}

		HDC hdcScreen = GetDC(0);

		if(hdcDesktop == 0)
		{
			hdcDesktop = CreateCompatibleDC(hdcScreen);
		}
		
		int cxDesktop = GetSystemMetrics(SM_CXSCREEN);
		int cyDesktop = GetSystemMetrics(SM_CYSCREEN);

		hbmDesktop = CreateCompatibleBitmap(hdcScreen, cxDesktop, cyDesktop);
		hbmDesktop = (HBITMAP) SelectObject(hdcDesktop, hbmDesktop);

		RECT r;
		SetRect(&r, 0, 0, cxDesktop, cyDesktop);
		FillRect(hdcDesktop, &r, GetSysColorBrush(COLOR_DESKTOP));

		char wallpaper[MAX_PATH_LENGTH] = { 0 };

		RegQueryStringValue(HKEY_CURRENT_USER,
			"Control Panel\\Desktop", "Wallpaper", wallpaper,
			MAX_PATH_LENGTH, "");

		if(wallpaper[0])
		{
			int wallpaperStyle;
			int tileWallpaper;

			char buffer[32];

			RegQueryStringValue(HKEY_CURRENT_USER, "Control Panel\\Desktop", "WallpaperStyle", buffer, 32, "0");
			wallpaperStyle = atoi(buffer);

			RegQueryStringValue(HKEY_CURRENT_USER, "Control Panel\\Desktop", "TileWallpaper", buffer, 32, "0");
			tileWallpaper = atoi(buffer);

			HBITMAP hbmWallpaper = (HBITMAP) LoadImage(0, wallpaper,
				IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);

			if(hbmWallpaper)
			{
				BITMAP bm;
				GetObject(hbmWallpaper, sizeof(BITMAP), &bm);

				int cxWallpaper = bm.bmWidth;
				int cyWallpaper = bm.bmHeight;

				HDC hdcWallpaper = CreateCompatibleDC(hdcScreen);
				hbmWallpaper = (HBITMAP) SelectObject(hdcWallpaper, hbmWallpaper);

				if(tileWallpaper)
				{
					TileBlt(hdcDesktop,
						0, 0,
						cxDesktop, cyDesktop,
						hdcWallpaper,
						0, 0,
						cxWallpaper, cyWallpaper,
						SRCCOPY);
				}
				else
				{
					if(wallpaperStyle == 2) // stretch
					{
						SetStretchBltMode(hdcDesktop, STRETCH_DELETESCANS);

						StretchBlt(hdcDesktop,
							0, 0,
							cxDesktop, cyDesktop,
							hdcWallpaper,
							0, 0,
							cxWallpaper, cyWallpaper,
							SRCCOPY);
					}
					else
					{
						BitBlt(hdcDesktop,
						(cxDesktop - cxWallpaper) / 2,
						(cyDesktop - cyWallpaper) / 2,
						cxWallpaper, cyWallpaper,
						hdcWallpaper,
						0, 0,
						SRCCOPY);
					}
				}

				hbmWallpaper = (HBITMAP) SelectObject(hdcWallpaper, hbmWallpaper);
				DeleteDC(hdcWallpaper);

				DeleteObject(hbmWallpaper);
			}
		}

		ReleaseDC(0, hdcScreen);
	}

	if(hdcDest)
	{
		BitBlt(hdcDest, xDest, yDest, cxDest, cyDest, hdcDesktop, xSrc, ySrc, SRCCOPY);
	}
}

// tile an image horizontally and vertically
void TileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	for(int y = 0; y < cyDest; y += cySrc)
	{
		for(int x = 0; x < cxDest; x += cxSrc)
		{
			BitBlt(hdcDest,
				xDest + x,
				yDest + y,
				min(cxDest - x, cxSrc),
				min(cyDest - y, cySrc),
				hdcSrc,
				xSrc,
				ySrc,
				rasterOp);
		}
	}
}

// tile an image horizontally
void HorizontalTileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	for(int x = 0; x < cxDest; x += cxSrc)
	{
		StretchBlt(hdcDest,
			xDest + x,
			yDest,
			min(cxDest - x, cxSrc),
			cyDest,
			hdcSrc,
			xSrc,
			ySrc,
			min(cxDest - x, cxSrc),
			cySrc,
			rasterOp);
	}
}

// tile an image vertically
void VerticalTileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	for(int y = 0; y < cyDest; y += cySrc)
	{
		StretchBlt(hdcDest,
			xDest,
			yDest + y,
			cxDest,
			min(cyDest - y, cySrc),
			hdcSrc,
			xSrc,
			ySrc,
			cxSrc,
			min(cyDest - y, cySrc),
			rasterOp);
	}
}

// multi-blt!
void MultiBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, int cxLeft, int cyTop, int cxRight, int cyBottom, int mode, DWORD rasterOp)
{
	if(cyTop > 0)
	{
		if(cxLeft > 0)
		{
			BitBlt(hdcDest,
				xDest,
				yDest,
				cxLeft,
				cyTop,
				hdcSrc,
				xSrc,
				ySrc,
				rasterOp);
		}

		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEVERTICAL)
		{
			StretchBlt(hdcDest,
				xDest + cxLeft,
				yDest,
				cxDest - cxLeft - cxRight,
				cyTop,
				hdcSrc,
				xSrc + cxLeft,
				ySrc,
				cxSrc - cxLeft - cxRight,
				cyTop,
				rasterOp);
		}
		else
		{
			HorizontalTileBlt(hdcDest,
				xDest + cxLeft,
				yDest,
				cxDest - cxLeft - cxRight,
				cyTop,
				hdcSrc,
				xSrc + cxLeft,
				ySrc,
				cxSrc - cxLeft - cxRight,
				cyTop,
				rasterOp);
		}

		if(cxRight > 0)
		{
			BitBlt(hdcDest,
				cxDest - cxRight,
				yDest,
				cxRight,
				cyTop,
				hdcSrc,
				cxSrc - cxRight,
				ySrc,
				rasterOp);
		}
	}

	if(cxLeft > 0)
	{
		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEHORIZONTAL)
		{
			StretchBlt(hdcDest,
				xDest,
				yDest + cyTop,
				cxLeft,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				xSrc,
				ySrc + cyTop,
				cxLeft,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
		else
		{
			VerticalTileBlt(hdcDest,
				xDest,
				yDest + cyTop,
				cxLeft,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				xSrc,
				ySrc + cyTop,
				cxLeft,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
	}

	if(mode == MULTIBLT_STRETCH)
	{
		StretchBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILE)
	{
		TileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILEHORIZONTAL)
	{
		HorizontalTileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILEVERTICAL)
	{
		VerticalTileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}

	if(cxRight > 0)
	{
		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEHORIZONTAL)
		{
			StretchBlt(hdcDest,
				cxDest - cxRight,
				yDest + cyTop,
				cxRight,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				ySrc + cyTop,
				cxRight,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
		else
		{
			VerticalTileBlt(hdcDest,
				cxDest - cxRight,
				yDest + cyTop,
				cxRight,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				ySrc + cyTop,
				cxRight,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
	}

	if(cyBottom > 0)
	{
		if(cxLeft > 0)
		{
			BitBlt(hdcDest,
				xDest,
				cyDest - cyBottom,
				cxLeft,
				cyBottom,
				hdcSrc,
				xSrc,
				cySrc - cyBottom,
				rasterOp);
		}

		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEVERTICAL)
		{
			StretchBlt(hdcDest,
				xDest + cxLeft,
				cyDest - cyBottom,
				cxDest - cxLeft - cxRight,
				cyBottom,
				hdcSrc,
				xSrc + cxLeft,
				cySrc - cyBottom,
				cxSrc - cxLeft - cxRight,
				cyBottom,
				rasterOp);
		}
		else
		{
			HorizontalTileBlt(hdcDest,
				xDest + cxLeft,
				cyDest - cyBottom,
				cxDest - cxLeft - cxRight,
				cyBottom,
				hdcSrc,
				xSrc + cxLeft,
				cySrc - cyBottom,
				cxSrc - cxLeft - cxRight,
				cyBottom,
				rasterOp);
		}

		if(cxRight > 0)
		{
			BitBlt(hdcDest,
				cxDest - cxRight,
				cyDest - cyBottom,
				cxRight,
				cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				cySrc - cyBottom,
				rasterOp);
		}
	}
}

// retrieve a string value from the registry
LPTSTR RegQueryStringValue(HKEY hKey, LPCTSTR pszSubKey, LPCTSTR pszValueName, LPTSTR pszBuffer, int nBufferSize, LPTSTR pszDefaultValue)
{
	DWORD dwDataType;
	DWORD dwDataSize = (DWORD) nBufferSize;
	HKEY hSubKey = NULL;
	LONG lErrorCode;
	
	if(pszSubKey)
	{
		lErrorCode = RegOpenKeyEx(hKey, pszSubKey, 0, KEY_QUERY_VALUE, &hSubKey);
		
		if(lErrorCode != ERROR_SUCCESS)
		{
			StringCchCopy(pszBuffer, nBufferSize, pszDefaultValue);
			return pszBuffer;
		}
		
		hKey = hSubKey;
	}
	
	lErrorCode = RegQueryValueEx(hKey, pszValueName, NULL, &dwDataType,
		(LPBYTE) pszBuffer, &dwDataSize);
	
	if(hSubKey)
		RegCloseKey(hSubKey);
		
	if(dwDataType != REG_SZ || lErrorCode != ERROR_SUCCESS)
		StringCchCopy(pszBuffer, nBufferSize, pszDefaultValue);
	
	return pszBuffer;
}

vector<string> split(string str, string sep)
{
	vector<string> substrs;
	string check = "";
	string keeper = "";
	int start = 0;
	int end = str.find(sep, start);
	
	while(end != string::npos)
	{
		if (sep == ";")
		{
			if ( end >= 3 )
				check = str.substr(end-3, 3);
			else
				check = "";

			if (stricmp(check.c_str(), "uot") == NULL || stricmp(check.c_str(), "&lt") == NULL || stricmp(check.c_str(), "&gt") == NULL || stricmp(check.c_str(), "amp") == NULL)
			{
				keeper.append(str.substr(start, end - start - sep.length() + 2));
			}
			else if (!keeper.empty())
			{
				keeper.append(str.substr(start, end - start - sep.length() + 1));
				substrs.push_back(keeper);
				keeper = "";
			}
			else
			{
				substrs.push_back(str.substr(start, end - start - sep.length() + 1));
			}
		}
		
		start = end + sep.length();
		end = str.find(sep, start);
	}

	if (!keeper.empty())
	{
		keeper.append(str.substr(start));
		substrs.push_back(keeper);
	}
	else
	{
		substrs.push_back(str.substr(start));
	}

	return substrs;
}

BOOL GetClipboardText(HWND hwndOwner, LPTSTR pszBuffer, UINT cchBuffer)
{
	HGLOBAL hMem;
	LPVOID pvMem;
	BOOL fResult = FALSE;

	if(IsClipboardFormatAvailable(CF_TEXT))
	{
		if(OpenClipboard(hwndOwner))
		{
			hMem = GetClipboardData(CF_TEXT);

			if(hMem)
			{
				pvMem = GlobalLock(hMem);
				lstrcpyn(pszBuffer, (LPCTSTR) pvMem, cchBuffer);
				GlobalUnlock(hMem);
				fResult = TRUE;
			}

			CloseClipboard();
		}
	}

	return fResult;
}

BOOL SetClipboardText(HWND hwndOwner, LPCTSTR pszText)
{
	UINT cbData;
	HGLOBAL hMem;
	LPVOID pvMem;
	BOOL fResult = FALSE;

	if(OpenClipboard(hwndOwner))
	{
		if(EmptyClipboard())
		{
			cbData = (lstrlen(pszText) + 1) * sizeof(TCHAR);
			hMem = GlobalAlloc(GMEM_MOVEABLE, cbData);

			if(hMem)
			{
				pvMem = GlobalLock(hMem);
				memcpy(pvMem, pszText, cbData);
				GlobalUnlock(hMem);
				fResult = SetClipboardData(CF_TEXT, hMem) ? TRUE : FALSE;

			}
		}

		CloseClipboard();
	}

	return fResult;
}

string IntToString(int input)
{
	char buffer[32] = {0};
	itoa( input, buffer, 10 );
	return (string) buffer;
}
